package javax.swing;

import java.awt.Rectangle;
import java.awt.Point;
import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleEditableText;
import javax.accessibility.AccessibleText;
import javax.accessibility.AccessibleAction;
import javax.accessibility.AccessibleValue;
import java.awt.event.ActionEvent;
import javax.swing.text.BadLocationException;
import javax.swing.text.AttributeSet;
import javax.swing.text.DocumentFilter;
import java.text.DecimalFormat;
import java.text.spi.NumberFormatProvider;
import java.text.NumberFormat;
import javax.swing.text.NumberFormatter;
import javax.swing.text.DefaultFormatterFactory;
import java.text.SimpleDateFormat;
import sun.util.locale.provider.LocaleResources;
import sun.util.locale.provider.LocaleProviderAdapter;
import java.text.spi.DateFormatProvider;
import java.util.Locale;
import java.text.DateFormat;
import javax.swing.text.DateFormatter;
import java.awt.Insets;
import java.awt.Dimension;
import java.awt.Container;
import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.awt.LayoutManager;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.text.ParseException;
import javax.swing.plaf.SpinnerUI;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.accessibility.Accessible;

public class JSpinner extends JComponent implements Accessible
{
    private static final String uiClassID = "SpinnerUI";
    private static final Action DISABLED_ACTION;
    private SpinnerModel model;
    private JComponent editor;
    private ChangeListener modelListener;
    private transient ChangeEvent changeEvent;
    private boolean editorExplicitlySet;
    
    public JSpinner(SpinnerModel model) {
        this.editorExplicitlySet = false;
        if (model == null) {
            throw new NullPointerException("model cannot be null");
        }
        this.model = model;
        this.editor = this.createEditor(model);
        this.setUIProperty("opaque", Boolean.valueOf(true));
        this.updateUI();
        return;
    }
    
    public JSpinner() {
        this(new SpinnerNumberModel());
    }
    
    public SpinnerUI getUI() {
        return (SpinnerUI)this.ui;
    }
    
    public void setUI(SpinnerUI ui) {
        super.setUI(ui);
    }
    
    @Override
    public String getUIClassID() {
        return "SpinnerUI";
    }
    
    @Override
    public void updateUI() {
        this.setUI((SpinnerUI)UIManager.getUI(this));
        this.invalidate();
    }
    
    protected JComponent createEditor(SpinnerModel spinnerModel) {
        if ((spinnerModel instanceof SpinnerDateModel)) {
            return new JSpinner.DateEditor(this);
        }
        if ((spinnerModel instanceof SpinnerListModel)) {
            return new JSpinner.ListEditor(this);
        }
        if ((spinnerModel instanceof SpinnerNumberModel)) {
            return new JSpinner.NumberEditor(this);
        }
        return new JSpinner.DefaultEditor(this);
    }
    
    public void setModel(SpinnerModel spinnerModel) {
        if (spinnerModel == null) {
            throw new IllegalArgumentException("null model");
        }
        if (!spinnerModel.equals(this.model)) {
            SpinnerModel model = this.model;
            this.model = spinnerModel;
            if (this.modelListener != null) {
                model.removeChangeListener(this.modelListener);
                this.model.addChangeListener(this.modelListener);
            }
            this.firePropertyChange("model", model, spinnerModel);
            if (!this.editorExplicitlySet) {
                this.setEditor(this.createEditor(spinnerModel));
                this.editorExplicitlySet = false;
            }
            this.repaint();
            this.revalidate();
        }
        return;
    }
    
    public SpinnerModel getModel() {
        return this.model;
    }
    
    public Object getValue() {
        return this.getModel().getValue();
    }
    
    public void setValue(Object value) {
        this.getModel().setValue(value);
    }
    
    public Object getNextValue() {
        return this.getModel().getNextValue();
    }
    
    public void addChangeListener(ChangeListener l) {
        if (this.modelListener == null) {
            this.modelListener = new JSpinner.ModelListener();
            this.getModel().addChangeListener(this.modelListener);
        }
        this.listenerList.add(ChangeListener.class, l);
    }
    
    public void removeChangeListener(ChangeListener l) {
        this.listenerList.remove(ChangeListener.class, l);
    }
    
    public ChangeListener[] getChangeListeners() {
        return (ChangeListener[])this.listenerList.getListeners(ChangeListener.class);
    }
    
    protected void fireStateChanged() {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ChangeListener.class) {
                if (this.changeEvent == null) {
                    this.changeEvent = new ChangeEvent(this);
                }
                ((ChangeListener)listenerList[i + 1]).stateChanged(this.changeEvent);
            }
        }
    }
    
    public Object getPreviousValue() {
        return this.getModel().getPreviousValue();
    }
    
    public void setEditor(JComponent component) {
        if (component == null) {
            throw new IllegalArgumentException("null editor");
        }
        if (!component.equals(this.editor)) {
            JComponent editor = this.editor;
            this.editor = component;
            if ((editor instanceof JSpinner.DefaultEditor)) {
                ((JSpinner.DefaultEditor)editor).dismiss(this);
            }
            this.editorExplicitlySet = true;
            this.firePropertyChange("editor", editor, component);
            this.revalidate();
            this.repaint();
        }
        return;
    }
    
    public JComponent getEditor() {
        return this.editor;
    }
    
    public void commitEdit() throws ParseException {
        JComponent editor = this.getEditor();
        if ((editor instanceof JSpinner.DefaultEditor)) {
            ((JSpinner.DefaultEditor)editor).commitEdit();
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("SpinnerUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JSpinner.AccessibleJSpinner();
        }
        return this.accessibleContext;
    }
    
    static {
        DISABLED_ACTION = new JSpinner.DisabledAction();
    }
    
    private class ModelListener implements ChangeListener, Serializable
    {
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            JSpinner.this.fireStateChanged();
        }
    }
    
    public static class DefaultEditor extends JPanel implements ChangeListener, PropertyChangeListener, LayoutManager
    {
        public DefaultEditor(JSpinner spinner) {
            super(null);
            JFormattedTextField comp = new JFormattedTextField();
            comp.setName("Spinner.formattedTextField");
            comp.setValue(spinner.getValue());
            comp.addPropertyChangeListener(this);
            comp.setEditable(false);
            comp.setInheritsPopupMenu(true);
            String toolTipText = spinner.getToolTipText();
            if (toolTipText != null) {
                comp.setToolTipText(toolTipText);
            }
            this.add(comp);
            this.setLayout(this);
            spinner.addChangeListener(this);
            ActionMap actionMap = comp.getActionMap();
            if (actionMap != null) {
                actionMap.put("increment", JSpinner.DISABLED_ACTION);
                actionMap.put("decrement", JSpinner.DISABLED_ACTION);
            }
        }
        
        public void dismiss(JSpinner spinner) {
            spinner.removeChangeListener(this);
        }
        
        public JSpinner getSpinner() {
            while (this != null) {
                if ((this instanceof JSpinner)) {
                    return (JSpinner)this;
                }
                this = this.getParent();
            }
            return null;
        }
        
        public JFormattedTextField getTextField() {
            return (JFormattedTextField)this.getComponent(0);
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            this.getTextField().setValue(((JSpinner)(JSpinner)changeEvent.getSource()).getValue());
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            JSpinner spinner = this.getSpinner();
            if (spinner == null) {
                return;
            }
            Object source = propertyChangeEvent.getSource();
            String propertyName = propertyChangeEvent.getPropertyName();
            if (source instanceof JFormattedTextField && "value".equals(propertyName)) {
                Object value = spinner.getValue();
                try {
                    spinner.setValue(this.getTextField().getValue());
                }
                catch (IllegalArgumentException ex) {
                    try {
                        ((JFormattedTextField)source).setValue(value);
                    }
                    catch (IllegalArgumentException ex2) {}
                }
            }
            return;
        }
        
        @Override
        public void addLayoutComponent(String s, Component component) {
        }
        
        @Override
        public void removeLayoutComponent(Component component) {
        }
        
        private Dimension insetSize(Container container) {
            Insets insets = container.getInsets();
            return new Dimension(insets.left + insets.right, insets.top + insets.bottom);
        }
        
        @Override
        public Dimension preferredLayoutSize(Container container) {
            Dimension insetSize = this.insetSize(container);
            if (container.getComponentCount() > 0) {
                Dimension preferredSize = this.getComponent(0).getPreferredSize();
                insetSize.width += preferredSize.width;
                insetSize.height += preferredSize.height;
                return insetSize;
            }
            return insetSize;
        }
        
        @Override
        public Dimension minimumLayoutSize(Container container) {
            Dimension insetSize = this.insetSize(container);
            if (container.getComponentCount() > 0) {
                Dimension minimumSize = this.getComponent(0).getMinimumSize();
                insetSize.width += minimumSize.width;
                insetSize.height += minimumSize.height;
                return insetSize;
            }
            return insetSize;
        }
        
        @Override
        public void layoutContainer(Container container) {
            if (container.getComponentCount() > 0) {
                Insets insets = container.getInsets();
                this.getComponent(0).setBounds(insets.left, insets.top, container.getWidth() - (insets.left + insets.right), container.getHeight() - (insets.top + insets.bottom));
            }
        }
        
        public void commitEdit() throws ParseException {
            this.getTextField().commitEdit();
        }
        
        @Override
        public int getBaseline(int width, int height) {
            super.getBaseline(width, height);
            Insets insets = this.getInsets();
            width = width - insets.left - insets.right;
            height = height - insets.top - insets.bottom;
            int baseline = this.getComponent(0).getBaseline(width, height);
            if (baseline >= 0) {
                return baseline + insets.top;
            }
            return -1;
        }
        
        @Override
        public Component.BaselineResizeBehavior getBaselineResizeBehavior() {
            return this.getComponent(0).getBaselineResizeBehavior();
        }
    }
    
    private static class DateEditorFormatter extends DateFormatter
    {
        private final SpinnerDateModel model;
        
        DateEditorFormatter(SpinnerDateModel model, DateFormat format) {
            super(format);
            this.model = model;
        }
        
        @Override
        public void setMinimum(Comparable start) {
            this.model.setStart(start);
        }
        
        @Override
        public Comparable getMinimum() {
            return this.model.getStart();
        }
        
        @Override
        public void setMaximum(Comparable end) {
            this.model.setEnd(end);
        }
        
        @Override
        public Comparable getMaximum() {
            return this.model.getEnd();
        }
    }
    
    public static class DateEditor extends JSpinner.DefaultEditor
    {
        private static String getDefaultPattern(Locale locale) {
            LocaleResources localeResources = LocaleProviderAdapter.getAdapter(DateFormatProvider.class, locale).getLocaleResources(locale);
            if (localeResources == null) {
                localeResources = LocaleProviderAdapter.forJRE().getLocaleResources(locale);
            }
            return localeResources.getDateTimePattern(3, 3, null);
        }
        
        public DateEditor(JSpinner spinner) {
            this(spinner, getDefaultPattern(spinner.getLocale()));
        }
        
        public DateEditor(JSpinner spinner, String pattern) {
            this(spinner, new SimpleDateFormat(pattern, spinner.getLocale()));
        }
        
        private DateEditor(JSpinner spinner, DateFormat dateFormat) {
            super(spinner);
            if (!(spinner.getModel() instanceof SpinnerDateModel)) {
                throw new IllegalArgumentException("model not a SpinnerDateModel");
            }
            SpinnerDateModel spinnerDateModel = (SpinnerDateModel)spinner.getModel();
            JSpinner.DateEditorFormatter defaultFormat = new JSpinner.DateEditorFormatter(spinnerDateModel, dateFormat);
            DefaultFormatterFactory formatterFactory = new DefaultFormatterFactory(defaultFormat);
            JFormattedTextField textField = this.getTextField();
            textField.setEditable(true);
            textField.setFormatterFactory(formatterFactory);
            try {
                textField.setColumns(Math.max(defaultFormat.valueToString(spinnerDateModel.getStart()).length(), defaultFormat.valueToString(spinnerDateModel.getEnd()).length()));
            }
            catch (ParseException ex) {}
            return;
        }
        
        public SimpleDateFormat getFormat() {
            return (SimpleDateFormat)((DateFormatter)(DateFormatter)this.getTextField().getFormatter()).getFormat();
        }
        
        public SpinnerDateModel getModel() {
            return (SpinnerDateModel)(SpinnerDateModel)this.getSpinner().getModel();
        }
    }
    
    private static class NumberEditorFormatter extends NumberFormatter
    {
        private final SpinnerNumberModel model;
        
        NumberEditorFormatter(SpinnerNumberModel model, NumberFormat format) {
            super(format);
            this.model = model;
            this.setValueClass(model.getValue().getClass());
        }
        
        @Override
        public void setMinimum(Comparable minimum) {
            this.model.setMinimum(minimum);
        }
        
        @Override
        public Comparable getMinimum() {
            return this.model.getMinimum();
        }
        
        @Override
        public void setMaximum(Comparable maximum) {
            this.model.setMaximum(maximum);
        }
        
        @Override
        public Comparable getMaximum() {
            return this.model.getMaximum();
        }
    }
    
    public static class NumberEditor extends JSpinner.DefaultEditor
    {
        private static String getDefaultPattern(Locale locale) {
            LocaleResources localeResources = LocaleProviderAdapter.getAdapter(NumberFormatProvider.class, locale).getLocaleResources(locale);
            if (localeResources == null) {
                localeResources = LocaleProviderAdapter.forJRE().getLocaleResources(locale);
            }
            return localeResources.getNumberPatterns()[0];
        }
        
        public NumberEditor(JSpinner spinner) {
            this(spinner, getDefaultPattern(spinner.getLocale()));
        }
        
        public NumberEditor(JSpinner spinner, String pattern) {
            this(spinner, new DecimalFormat(pattern));
        }
        
        private NumberEditor(JSpinner spinner, DecimalFormat decimalFormat) {
            super(spinner);
            if (!(spinner.getModel() instanceof SpinnerNumberModel)) {
                throw new IllegalArgumentException("model not a SpinnerNumberModel");
            }
            SpinnerNumberModel spinnerNumberModel = (SpinnerNumberModel)spinner.getModel();
            JSpinner.NumberEditorFormatter defaultFormat = new JSpinner.NumberEditorFormatter(spinnerNumberModel, decimalFormat);
            DefaultFormatterFactory formatterFactory = new DefaultFormatterFactory(defaultFormat);
            JFormattedTextField textField = this.getTextField();
            textField.setEditable(true);
            textField.setFormatterFactory(formatterFactory);
            textField.setHorizontalAlignment(4);
            try {
                textField.setColumns(Math.max(defaultFormat.valueToString(spinnerNumberModel.getMinimum()).length(), defaultFormat.valueToString(spinnerNumberModel.getMaximum()).length()));
            }
            catch (ParseException ex) {}
            return;
        }
        
        public DecimalFormat getFormat() {
            return (DecimalFormat)((NumberFormatter)(NumberFormatter)this.getTextField().getFormatter()).getFormat();
        }
        
        public SpinnerNumberModel getModel() {
            return (SpinnerNumberModel)(SpinnerNumberModel)this.getSpinner().getModel();
        }
    }
    
    public static class ListEditor extends JSpinner.DefaultEditor
    {
        public ListEditor(JSpinner spinner) {
            super(spinner);
            if (!(spinner.getModel() instanceof SpinnerListModel)) {
                throw new IllegalArgumentException("model not a SpinnerListModel");
            }
            this.getTextField().setEditable(true);
            this.getTextField().setFormatterFactory(new DefaultFormatterFactory(new JSpinner.ListEditor.ListFormatter()));
            return;
        }
        
        public SpinnerListModel getModel() {
            return (SpinnerListModel)(SpinnerListModel)this.getSpinner().getModel();
        }
        
        private class ListFormatter extends JFormattedTextField.AbstractFormatter
        {
            private DocumentFilter filter;
            
            @Override
            public String valueToString(Object o) throws ParseException {
                if (o == null) {
                    return "";
                }
                return o.toString();
            }
            
            @Override
            public Object stringToValue(String s) throws ParseException {
                return s;
            }
            
            @Override
            protected DocumentFilter getDocumentFilter() {
                if (this.filter == null) {
                    this.filter = new JSpinner.ListEditor.ListFormatter.Filter();
                }
                return this.filter;
            }
            
            private class Filter extends DocumentFilter
            {
                @Override
                public void replace(DocumentFilter.FilterBypass fb, int offset, int length, String s, AttributeSet attrs) throws BadLocationException {
                    if (s != null && offset + length == fb.getDocument().getLength()) {
                        Object nextMatch = JSpinner.ListEditor.this.getModel().findNextMatch(fb.getDocument().getText(0, offset) + s);
                        String s2 = (nextMatch != null) ? nextMatch.toString() : null;
                        if (s2 != null) {
                            fb.remove(0, offset + length);
                            fb.insertString(0, s2, null);
                            JSpinner.ListEditor.ListFormatter.this.getFormattedTextField().select(offset + s.length(), s2.length());
                            return;
                        }
                    }
                    super.replace(fb, offset, length, s, attrs);
                }
                
                @Override
                public void insertString(DocumentFilter.FilterBypass filterBypass, int n, String s, AttributeSet set) throws BadLocationException {
                    this.replace(filterBypass, n, 0, s, set);
                }
            }
        }
    }
    
    private static class DisabledAction implements Action
    {
        @Override
        public Object getValue(String s) {
            return null;
        }
        
        @Override
        public void putValue(String s, Object o) {
        }
        
        @Override
        public void setEnabled(boolean b) {
        }
        
        @Override
        public boolean isEnabled() {
            return false;
        }
        
        @Override
        public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        }
        
        @Override
        public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
        }
    }
    
    protected class AccessibleJSpinner extends JComponent.AccessibleJComponent implements AccessibleValue, AccessibleAction, AccessibleText, AccessibleEditableText, ChangeListener
    {
        private Object oldModelValue;
        
        protected AccessibleJSpinner() {
            this.oldModelValue = null;
            this.oldModelValue = JSpinner.this.model.getValue();
            JSpinner.this.addChangeListener(this);
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            if (changeEvent == null) {
                throw new NullPointerException();
            }
            Object value = JSpinner.this.model.getValue();
            this.firePropertyChange("AccessibleValue", this.oldModelValue, value);
            this.firePropertyChange("AccessibleText", null, Integer.valueOf(0));
            this.oldModelValue = value;
            return;
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.SPIN_BOX;
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            if (JSpinner.this.editor.getAccessibleContext() != null) {
                return 1;
            }
            return 0;
        }
        
        @Override
        public Accessible getAccessibleChild(int n) {
            if (n != 0) {
                return null;
            }
            if (JSpinner.this.editor.getAccessibleContext() != null) {
                return (Accessible)JSpinner.this.editor;
            }
            return null;
        }
        
        @Override
        public AccessibleAction getAccessibleAction() {
            return this;
        }
        
        @Override
        public AccessibleText getAccessibleText() {
            return this;
        }
        
        private AccessibleContext getEditorAccessibleContext() {
            if ((JSpinner.this.editor instanceof JSpinner.DefaultEditor)) {
                JFormattedTextField textField = ((JSpinner.DefaultEditor)JSpinner.this.editor).getTextField();
                if (textField != null) {
                    return textField.getAccessibleContext();
                }
            }
            else if ((JSpinner.this.editor instanceof Accessible)) {
                return JSpinner.this.editor.getAccessibleContext();
            }
            return null;
        }
        
        private AccessibleText getEditorAccessibleText() {
            AccessibleContext editorAccessibleContext = this.getEditorAccessibleContext();
            if (editorAccessibleContext != null) {
                return editorAccessibleContext.getAccessibleText();
            }
            return null;
        }
        
        private AccessibleEditableText getEditorAccessibleEditableText() {
            AccessibleText editorAccessibleText = this.getEditorAccessibleText();
            if ((editorAccessibleText instanceof AccessibleEditableText)) {
                return (AccessibleEditableText)editorAccessibleText;
            }
            return null;
        }
        
        @Override
        public AccessibleValue getAccessibleValue() {
            return this;
        }
        
        @Override
        public Number getCurrentAccessibleValue() {
            Object value = JSpinner.this.model.getValue();
            if ((value instanceof Number)) {
                return (Number)value;
            }
            return null;
        }
        
        @Override
        public boolean setCurrentAccessibleValue(Number value) {
            try {
                JSpinner.this.model.setValue(value);
                return true;
            }
            catch (IllegalArgumentException ex) {
                return false;
            }
        }
        
        @Override
        public Number getMinimumAccessibleValue() {
            if ((JSpinner.this.model instanceof SpinnerNumberModel)) {
                Comparable<Object> minimum = ((SpinnerNumberModel)JSpinner.this.model).getMinimum();
                if ((minimum instanceof Number)) {
                    return (Number)minimum;
                }
            }
            return null;
        }
        
        @Override
        public Number getMaximumAccessibleValue() {
            if ((JSpinner.this.model instanceof SpinnerNumberModel)) {
                Comparable<Object> maximum = ((SpinnerNumberModel)JSpinner.this.model).getMaximum();
                if ((maximum instanceof Number)) {
                    return (Number)maximum;
                }
            }
            return null;
        }
        
        @Override
        public int getAccessibleActionCount() {
            return 2;
        }
        
        @Override
        public String getAccessibleActionDescription(int n) {
            if (n == 0) {
                return AccessibleAction.INCREMENT;
            }
            if (n == 1) {
                return AccessibleAction.DECREMENT;
            }
            return null;
        }
        
        @Override
        public boolean doAccessibleAction(int n) {
            if (n < 0 || n > 1) {
                return false;
            }
            Object value;
            if (n == 0) {
                value = JSpinner.this.getNextValue();
            }
            else {
                value = JSpinner.this.getPreviousValue();
            }
            try {
                JSpinner.this.model.setValue(value);
                return true;
            }
            catch (IllegalArgumentException ex) {
                return false;
            }
        }
        
        private boolean sameWindowAncestor(Component component, Component component2) {
            return component != null && component2 != null && SwingUtilities.getWindowAncestor(component) == SwingUtilities.getWindowAncestor(component2);
        }
        
        @Override
        public int getIndexAtPoint(Point point) {
            AccessibleText editorAccessibleText = this.getEditorAccessibleText();
            if (editorAccessibleText != null && this.sameWindowAncestor(JSpinner.this, JSpinner.this.editor)) {
                Point convertPoint = SwingUtilities.convertPoint(JSpinner.this, point, JSpinner.this.editor);
                if (convertPoint != null) {
                    return editorAccessibleText.getIndexAtPoint(convertPoint);
                }
            }
            return -1;
        }
        
        @Override
        public Rectangle getCharacterBounds(int n) {
            AccessibleText editorAccessibleText = this.getEditorAccessibleText();
            if (editorAccessibleText != null) {
                Rectangle characterBounds = editorAccessibleText.getCharacterBounds(n);
                if (characterBounds != null && this.sameWindowAncestor(JSpinner.this, JSpinner.this.editor)) {
                    return SwingUtilities.convertRectangle(JSpinner.this.editor, characterBounds, JSpinner.this);
                }
            }
            return null;
        }
        
        @Override
        public int getCharCount() {
            AccessibleText editorAccessibleText = this.getEditorAccessibleText();
            if (editorAccessibleText != null) {
                return editorAccessibleText.getCharCount();
            }
            return -1;
        }
        
        @Override
        public int getCaretPosition() {
            AccessibleText editorAccessibleText = this.getEditorAccessibleText();
            if (editorAccessibleText != null) {
                return editorAccessibleText.getCaretPosition();
            }
            return -1;
        }
        
        @Override
        public String getAtIndex(int n, int n2) {
            AccessibleText editorAccessibleText = this.getEditorAccessibleText();
            if (editorAccessibleText != null) {
                return editorAccessibleText.getAtIndex(n, n2);
            }
            return null;
        }
        
        @Override
        public String getAfterIndex(int n, int n2) {
            AccessibleText editorAccessibleText = this.getEditorAccessibleText();
            if (editorAccessibleText != null) {
                return editorAccessibleText.getAfterIndex(n, n2);
            }
            return null;
        }
        
        @Override
        public String getBeforeIndex(int n, int n2) {
            AccessibleText editorAccessibleText = this.getEditorAccessibleText();
            if (editorAccessibleText != null) {
                return editorAccessibleText.getBeforeIndex(n, n2);
            }
            return null;
        }
        
        @Override
        public AttributeSet getCharacterAttribute(int n) {
            AccessibleText editorAccessibleText = this.getEditorAccessibleText();
            if (editorAccessibleText != null) {
                return editorAccessibleText.getCharacterAttribute(n);
            }
            return null;
        }
        
        @Override
        public int getSelectionStart() {
            AccessibleText editorAccessibleText = this.getEditorAccessibleText();
            if (editorAccessibleText != null) {
                return editorAccessibleText.getSelectionStart();
            }
            return -1;
        }
        
        @Override
        public int getSelectionEnd() {
            AccessibleText editorAccessibleText = this.getEditorAccessibleText();
            if (editorAccessibleText != null) {
                return editorAccessibleText.getSelectionEnd();
            }
            return -1;
        }
        
        @Override
        public String getSelectedText() {
            AccessibleText editorAccessibleText = this.getEditorAccessibleText();
            if (editorAccessibleText != null) {
                return editorAccessibleText.getSelectedText();
            }
            return null;
        }
        
        @Override
        public void setTextContents(String textContents) {
            AccessibleEditableText editorAccessibleEditableText = this.getEditorAccessibleEditableText();
            if (editorAccessibleEditableText != null) {
                editorAccessibleEditableText.setTextContents(textContents);
            }
        }
        
        @Override
        public void insertTextAtIndex(int n, String s) {
            AccessibleEditableText editorAccessibleEditableText = this.getEditorAccessibleEditableText();
            if (editorAccessibleEditableText != null) {
                editorAccessibleEditableText.insertTextAtIndex(n, s);
            }
        }
        
        @Override
        public String getTextRange(int n, int n2) {
            AccessibleEditableText editorAccessibleEditableText = this.getEditorAccessibleEditableText();
            if (editorAccessibleEditableText != null) {
                return editorAccessibleEditableText.getTextRange(n, n2);
            }
            return null;
        }
        
        @Override
        public void delete(int n, int n2) {
            AccessibleEditableText editorAccessibleEditableText = this.getEditorAccessibleEditableText();
            if (editorAccessibleEditableText != null) {
                editorAccessibleEditableText.delete(n, n2);
            }
        }
        
        @Override
        public void cut(int n, int n2) {
            AccessibleEditableText editorAccessibleEditableText = this.getEditorAccessibleEditableText();
            if (editorAccessibleEditableText != null) {
                editorAccessibleEditableText.cut(n, n2);
            }
        }
        
        @Override
        public void paste(int n) {
            AccessibleEditableText editorAccessibleEditableText = this.getEditorAccessibleEditableText();
            if (editorAccessibleEditableText != null) {
                editorAccessibleEditableText.paste(n);
            }
        }
        
        @Override
        public void replaceText(int n, int n2, String s) {
            AccessibleEditableText editorAccessibleEditableText = this.getEditorAccessibleEditableText();
            if (editorAccessibleEditableText != null) {
                editorAccessibleEditableText.replaceText(n, n2, s);
            }
        }
        
        @Override
        public void selectText(int n, int n2) {
            AccessibleEditableText editorAccessibleEditableText = this.getEditorAccessibleEditableText();
            if (editorAccessibleEditableText != null) {
                editorAccessibleEditableText.selectText(n, n2);
            }
        }
        
        @Override
        public void setAttributes(int n, int n2, AttributeSet set) {
            AccessibleEditableText editorAccessibleEditableText = this.getEditorAccessibleEditableText();
            if (editorAccessibleEditableText != null) {
                editorAccessibleEditableText.setAttributes(n, n2, set);
            }
        }
    }
}
